@@ -85,7 +85,7 @@ fn is_broken_pipe(err: &Error) -> bool {
 ///
 /// This tool behaves like unix `cut` with a few exceptions:
 ///
-/// * `delimiter` is a fixed substring by default, and a regex with `-R`
+/// * `delimiter` is a regex by default and a fixed substring with `-L`
 /// * `header-fields` allows for specifying a literal or a regex to match header names to select columns
 /// * both `header-fields` and `fields` order dictate the order of the output columns
 /// * input files (not stdin) are automatically compressed
@@ -123,14 +123,13 @@ struct Opts {
     #[structopt(short = "o", long)]
     output: Option<PathBuf>,
 
-    /// Delimiter to use on input files, this is a substring literal by default. To treat it as a regex add the
-    /// `-R` flag.
-    #[structopt(short = "d", long, default_value = "\t")]
+    /// Delimiter to use on input files, this is a substring literal by default. To treat it as a literal add the `-L` flag.
+    #[structopt(short = "d", long, default_value = r"\s+")]
     delimiter: String,
 
-    /// Treat the delimiter as a regex
-    #[structopt(short = "R", long)]
-    delim_is_regex: bool,
+    /// Treat the delimiter as a string literal. This can significantly improve performance, especially for single byte delimiters.
+    #[structopt(short = "L", long)]
+    delim_is_literal: bool,
 
     /// Delimiter string to use on outputs
     #[structopt(short = "D", long, default_value = "\t")]
@@ -140,10 +139,20 @@ struct Opts {
     #[structopt(short = "f", long)]
     fields: Option<String>,
 
+    /// Fields to exclude from the output, ex: 3,9-11,15-. Exclude fields are 1 based and inclusive.
+    /// Exclude fields take precedence over `fields`.
+    #[structopt(short = "e", long)]
+    exclude: Option<String>,
+
+    /// Headers to exclude from the output, ex: '^badfield.*$`. This is a string literal by default.
+    /// Add the `-r` flag to treat as a regex.
+    #[structopt(short = "E", long)]
+    exclude_header: Option<Vec<Regex>>,
+
     /// A string literal or regex to select headers, ex: '^is_.*$`. This is a string literal
-    /// by deafult. add the `-r` flag to treat it as a regex.
+    /// by default. add the `-r` flag to treat it as a regex.
     #[structopt(short = "F", long)]
-    header_fields: Option<Vec<Regex>>,
+    header_field: Option<Vec<Regex>>,
 
     /// Treat the header_fields as regexs instead of string literals
     #[structopt(short = "r", long)]
@@ -153,7 +162,7 @@ struct Opts {
     #[structopt(short = "z", long)]
     try_decompress: bool,
 
-    /// Disallow the posibility of using mmap
+    /// Disallow the possibility of using mmap
     #[structopt(long)]
     no_mmap: bool,
 
@@ -201,7 +210,7 @@ fn main() -> Result<()> {
     conf_builder.mmap(mmap);
     conf_builder.delimiter(&opts.delimiter.as_bytes());
     conf_builder.output_delimiter(&opts.output_delimiter.as_bytes());
-    conf_builder.is_regex_parser(opts.delim_is_regex);
+    conf_builder.is_regex_parser(!opts.delim_is_literal);
     conf_builder.try_decompress(opts.try_decompress);
     let conf = conf_builder.build();
 
@@ -236,7 +245,7 @@ fn run<W: Write>(
     };
 
     // Parser the fields in the context of the files being looked at
-    let (extra, fields) = match (&opts.fields, &opts.header_fields) {
+    let (mut extra, fields) = match (&opts.fields, &opts.header_field) {
         (Some(field_list), Some(header_fields)) => {
             let first_line = input.peek_first_line()?;
             let mut fields = FieldRange::from_list(field_list)?;
@@ -261,12 +270,54 @@ fn run<W: Write>(
             )?;
             (Some(first_line), fields)
         }
-        (None, None) => {
-            eprintln!("Must select one or both `fields` and 'header-fields`.");
-            exit(1);
-        }
+        (None, None) => (None, FieldRange::from_list("1-")?),
     };
 
+    let fields = match (&opts.exclude, &opts.exclude_header) {
+        (Some(exclude), Some(exclude_header)) => {
+            let exclude = FieldRange::from_list(exclude)?;
+            let fields = FieldRange::exclude(fields, exclude);
+            let first_line = if let Some(first_line) = extra {
+                first_line
+            } else {
+                input.peek_first_line()?
+            };
+            let exclude_headers = FieldRange::from_header_list(
+                &exclude_header,
+                first_line.as_bytes(),
+                &delim,
+                opts.header_is_regex,
+            )?;
+            extra = Some(first_line);
+            FieldRange::exclude(fields, exclude_headers)
+        }
+        (Some(exclude), None) => {
+            let exclude = FieldRange::from_list(exclude)?;
+            FieldRange::exclude(fields, exclude)
+        }
+        (None, Some(exclude_header)) => {
+            let first_line = if let Some(first_line) = extra {
+                first_line
+            } else {
+                input.peek_first_line()?
+            };
+            let exclude_headers = FieldRange::from_header_list(
+                &exclude_header,
+                first_line.as_bytes(),
+                &delim,
+                opts.header_is_regex,
+            )?;
+            extra = Some(first_line);
+            FieldRange::exclude(fields, exclude_headers)
+        }
+        (None, None) => fields,
+    };
+
+    // No point processing empty fields
+    if fields.is_empty() {
+        return Ok(());
+    }
+
     match &delim {
         RegexOrStr::Regex(regex) => {
             let mut core = Core::new(
@@ -306,19 +357,22 @@ mod test {
         output_file: impl AsRef<Path>,
         fields: &str,
         no_mmap: bool,
+        delimiter: &str,
     ) -> Opts {
         Opts {
             input: vec![input_file.as_ref().to_path_buf()],
             output: Some(output_file.as_ref().to_path_buf()),
-            delimiter: String::from(r"\s+"),
-            delim_is_regex: true,
+            delimiter: delimiter.to_string(),
+            delim_is_literal: false,
             output_delimiter: "\t".to_owned(),
             fields: Some(fields.to_owned()),
-            header_fields: None,
+            header_field: None,
             header_is_regex: true,
             try_decompress: false,
             no_mmap,
             crlf: false,
+            exclude: None,
+            exclude_header: None,
         }
     }
 
@@ -328,19 +382,49 @@ mod test {
         output_file: impl AsRef<Path>,
         fields: &str,
         no_mmap: bool,
+        delimiter: &str,
     ) -> Opts {
         Opts {
             input: vec![input_file.as_ref().to_path_buf()],
             output: Some(output_file.as_ref().to_path_buf()),
-            delimiter: String::from(FOURSPACE),
-            delim_is_regex: false,
+            delimiter: delimiter.to_string(),
+            delim_is_literal: true,
             output_delimiter: "\t".to_owned(),
             fields: Some(fields.to_owned()),
-            header_fields: None,
+            header_field: None,
             header_is_regex: true,
             try_decompress: false,
             no_mmap,
             crlf: false,
+            exclude: None,
+            exclude_header: None,
+        }
+    }
+
+    /// Build a set of opts for testing
+    fn build_opts_generic(
+        input_file: impl AsRef<Path>,
+        output_file: impl AsRef<Path>,
+        fields: &str,
+        exclude: Option<String>,
+        no_mmap: bool,
+        delimiter: &str,
+        delim_is_literal: bool,
+    ) -> Opts {
+        Opts {
+            input: vec![input_file.as_ref().to_path_buf()],
+            output: Some(output_file.as_ref().to_path_buf()),
+            delimiter: delimiter.to_string(),
+            delim_is_literal,
+            output_delimiter: "\t".to_owned(),
+            fields: Some(fields.to_owned()),
+            header_field: None,
+            header_is_regex: true,
+            try_decompress: false,
+            no_mmap,
+            crlf: false,
+            exclude,
+            exclude_header: None,
         }
     }
 
@@ -374,7 +458,7 @@ mod test {
     fn run_wrapper<P: AsRef<Path>>(input: P, output: P, opts: &Opts) {
         let conf = CoreConfigBuilder::new()
             .delimiter(opts.delimiter.as_bytes())
-            .is_regex_parser(opts.delim_is_regex)
+            .is_regex_parser(!opts.delim_is_literal)
             .mmap(if opts.no_mmap {
                 MmapChoice::never()
             } else {
@@ -397,17 +481,191 @@ mod test {
     const FOURSPACE: &str = "    ";
 
     #[rstest]
-    #[rustfmt::skip::macros(vec)]
-    fn test_read_single_values(#[values(true, false)] no_mmap: bool) {
+    fn test_exclude_one(
+        #[values(true, false)] no_mmap: bool,
+        #[values(r" ", "  ")] hck_delim: &str,
+        #[values(true, false)] delim_is_literal: bool,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "1", no_mmap);
+        let opts = build_opts_generic(
+            &input_file,
+            &output_file,
+            "1,3",
+            Some(String::from("3")),
+            no_mmap,
+            hck_delim,
+            delim_is_literal,
+        );
+        let data = vec![vec!["a", "b", "c"], vec!["1", "2", "3"]];
+        write_file(&input_file, data, hck_delim);
+        run_wrapper(&input_file, &output_file, &opts);
+        let filtered = read_tsv(output_file);
+
+        assert_eq!(filtered, vec![vec!["a",], vec!["1"]]);
+    }
+
+    #[rstest]
+    fn test_exclude_range_overlap_front(
+        #[values(true, false)] no_mmap: bool,
+        #[values(r" ", "  ")] hck_delim: &str,
+        #[values(true, false)] delim_is_literal: bool,
+    ) {
+        let tmp = TempDir::new().unwrap();
+        let input_file = tmp.path().join("input.txt");
+        let output_file = tmp.path().join("output.txt");
+        let opts = build_opts_generic(
+            &input_file,
+            &output_file,
+            "3-",
+            Some(String::from("-5")),
+            no_mmap,
+            hck_delim,
+            delim_is_literal,
+        );
+        let data = vec![
+            vec!["a", "b", "c", "d", "e", "f"],
+            vec!["1", "2", "3", "4", "5", "6"],
+        ];
+        write_file(&input_file, data, hck_delim);
+        run_wrapper(&input_file, &output_file, &opts);
+        let filtered = read_tsv(output_file);
+
+        assert_eq!(filtered, vec![vec!["f",], vec!["6"]]);
+    }
+
+    #[rstest]
+    fn test_exclude_range_overlap_back(
+        #[values(true, false)] no_mmap: bool,
+        #[values(r" ", "  ")] hck_delim: &str,
+        #[values(true, false)] delim_is_literal: bool,
+    ) {
+        let tmp = TempDir::new().unwrap();
+        let input_file = tmp.path().join("input.txt");
+        let output_file = tmp.path().join("output.txt");
+        let opts = build_opts_generic(
+            &input_file,
+            &output_file,
+            "2-5",
+            Some(String::from("3-")),
+            no_mmap,
+            hck_delim,
+            delim_is_literal,
+        );
+        let data = vec![
+            vec!["a", "b", "c", "d", "e", "f"],
+            vec!["1", "2", "3", "4", "5", "6"],
+        ];
+        write_file(&input_file, data, hck_delim);
+        run_wrapper(&input_file, &output_file, &opts);
+        let filtered = read_tsv(output_file);
+
+        assert_eq!(filtered, vec![vec!["b",], vec!["2"]]);
+    }
+
+    #[rstest]
+    fn test_exclude_range_split_fields(
+        #[values(true, false)] no_mmap: bool,
+        #[values(r" ", "  ")] hck_delim: &str,
+        #[values(true, false)] delim_is_literal: bool,
+    ) {
+        let tmp = TempDir::new().unwrap();
+        let input_file = tmp.path().join("input.txt");
+        let output_file = tmp.path().join("output.txt");
+        let opts = build_opts_generic(
+            &input_file,
+            &output_file,
+            "1-",
+            Some(String::from("3-5")),
+            no_mmap,
+            hck_delim,
+            delim_is_literal,
+        );
+        let data = vec![
+            vec!["a", "b", "c", "d", "e", "f"],
+            vec!["1", "2", "3", "4", "5", "6"],
+        ];
+        write_file(&input_file, data, hck_delim);
+        run_wrapper(&input_file, &output_file, &opts);
+        let filtered = read_tsv(output_file);
+
+        assert_eq!(filtered, vec![vec!["a", "b", "f"], vec!["1", "2", "6"]]);
+    }
+
+    #[rstest]
+    fn test_exclude_range_all(
+        #[values(true, false)] no_mmap: bool,
+        #[values(r" ", "  ")] hck_delim: &str,
+        #[values(true, false)] delim_is_literal: bool,
+    ) {
+        let tmp = TempDir::new().unwrap();
+        let input_file = tmp.path().join("input.txt");
+        let output_file = tmp.path().join("output.txt");
+        let opts = build_opts_generic(
+            &input_file,
+            &output_file,
+            "4,3",
+            Some(String::from("2-5")),
+            no_mmap,
+            hck_delim,
+            delim_is_literal,
+        );
+        let data = vec![
+            vec!["a", "b", "c", "d", "e", "f"],
+            vec!["1", "2", "3", "4", "5", "6"],
+        ];
+        write_file(&input_file, data, hck_delim);
+        run_wrapper(&input_file, &output_file, &opts);
+        let filtered = read_tsv(output_file);
+
+        assert!(filtered.is_empty());
+    }
+
+    #[rstest]
+    fn test_exclude_range_split_fields_reorder(
+        #[values(true, false)] no_mmap: bool,
+        #[values(r" ", "  ")] hck_delim: &str,
+        #[values(true, false)] delim_is_literal: bool,
+    ) {
+        let tmp = TempDir::new().unwrap();
+        let input_file = tmp.path().join("input.txt");
+        let output_file = tmp.path().join("output.txt");
+        let opts = build_opts_generic(
+            &input_file,
+            &output_file,
+            "4-6,1-3",
+            Some(String::from("3-5")),
+            no_mmap,
+            hck_delim,
+            delim_is_literal,
+        );
+        let data = vec![
+            vec!["a", "b", "c", "d", "e", "f"],
+            vec!["1", "2", "3", "4", "5", "6"],
+        ];
+        write_file(&input_file, data, hck_delim);
+        run_wrapper(&input_file, &output_file, &opts);
+        let filtered = read_tsv(output_file);
+
+        assert_eq!(filtered, vec![vec!["f", "a", "b"], vec!["6", "1", "2"]]);
+    }
+
+    #[rstest]
+    #[rustfmt::skip::macros(vec)]
+    fn test_read_single_values(
+        #[values(true, false)] no_mmap: bool,
+        #[values("    ", " ")] hck_delim: &str,
+    ) {
+        let tmp = TempDir::new().unwrap();
+        let input_file = tmp.path().join("input.txt");
+        let output_file = tmp.path().join("output.txt");
+        let opts = build_opts(&input_file, &output_file, "1", no_mmap, hck_delim);
         let data = vec![
             vec!["a", "b", "c"],
             vec!["1", "2", "3"],
         ];
-        write_file(&input_file, data, FOURSPACE);
+        write_file(&input_file, data, hck_delim);
         run_wrapper(&input_file, &output_file, &opts);
         let filtered = read_tsv(output_file);
 
@@ -415,11 +673,14 @@ mod test {
     }
 
     #[rstest]
-    fn test_read_several_single_values(#[values(true, false)] no_mmap: bool) {
+    fn test_read_several_single_values(
+        #[values(true, false)] no_mmap: bool,
+        #[values(r"\s+")] hck_delim: &str,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "1,3", no_mmap);
+        let opts = build_opts(&input_file, &output_file, "1,3", no_mmap, hck_delim);
         let data = vec![vec!["a", "b", "c"], vec!["1", "2", "3"]];
         write_file(&input_file, data, FOURSPACE);
         run_wrapper(&input_file, &output_file, &opts);
@@ -429,11 +690,14 @@ mod test {
     }
 
     #[rstest]
-    fn test_read_several_single_values_with_invalid_utf8(#[values(true, false)] no_mmap: bool) {
+    fn test_read_several_single_values_with_invalid_utf8(
+        #[values(true, false)] no_mmap: bool,
+        #[values(r"\s+")] hck_delim: &str,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "1,3", no_mmap);
+        let opts = build_opts(&input_file, &output_file, "1,3", no_mmap, hck_delim);
         let bad_str = unsafe { String::from_utf8_unchecked(b"a\xED\xA0\x80z".to_vec()) };
         let data = vec![vec![bad_str.as_str(), "b", "c"], vec!["1", "2", "3"]];
         write_file(&input_file, data, FOURSPACE);
@@ -444,11 +708,14 @@ mod test {
     }
 
     #[rstest]
-    fn test_read_single_range(#[values(true, false)] no_mmap: bool) {
+    fn test_read_single_range(
+        #[values(true, false)] no_mmap: bool,
+        #[values(r"\s+")] hck_delim: &str,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "2-", no_mmap);
+        let opts = build_opts(&input_file, &output_file, "2-", no_mmap, hck_delim);
         let data = vec![vec!["a", "b", "c", "d"], vec!["1", "2", "3", "4"]];
         write_file(&input_file, data, FOURSPACE);
         run_wrapper(&input_file, &output_file, &opts);
@@ -458,11 +725,14 @@ mod test {
     }
 
     #[rstest]
-    fn test_read_serveral_range(#[values(true, false)] no_mmap: bool) {
+    fn test_read_serveral_range(
+        #[values(true, false)] no_mmap: bool,
+        #[values(r"\s+")] hck_delim: &str,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "2-4,6-", no_mmap);
+        let opts = build_opts(&input_file, &output_file, "2-4,6-", no_mmap, hck_delim);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -478,11 +748,14 @@ mod test {
     }
 
     #[rstest]
-    fn test_read_mixed_fields1(#[values(true, false)] no_mmap: bool) {
+    fn test_read_mixed_fields1(
+        #[values(true, false)] no_mmap: bool,
+        #[values(r"\s+")] hck_delim: &str,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "2,4-", no_mmap);
+        let opts = build_opts(&input_file, &output_file, "2,4-", no_mmap, hck_delim);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -498,11 +771,14 @@ mod test {
     }
 
     #[rstest]
-    fn test_read_mixed_fields2(#[values(true, false)] no_mmap: bool) {
+    fn test_read_mixed_fields2(
+        #[values(true, false)] no_mmap: bool,
+        #[values(r"\s+")] hck_delim: &str,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "-4,7", no_mmap);
+        let opts = build_opts(&input_file, &output_file, "-4,7", no_mmap, hck_delim);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -518,11 +794,14 @@ mod test {
     }
 
     #[rstest]
-    fn test_read_no_delimis_found(#[values(true, false)] no_mmap: bool) {
+    fn test_read_no_delimis_found(
+        #[values(true, false)] no_mmap: bool,
+        #[values(r"\s+")] hck_delim: &str,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "-4,7", no_mmap);
+        let opts = build_opts(&input_file, &output_file, "-4,7", no_mmap, hck_delim);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -537,11 +816,11 @@ mod test {
     }
 
     #[rstest]
-    fn test_read_over_end(#[values(true, false)] no_mmap: bool) {
+    fn test_read_over_end(#[values(true, false)] no_mmap: bool, #[values(r"\s+")] hck_delim: &str) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "-4,8,11-", no_mmap);
+        let opts = build_opts(&input_file, &output_file, "-4,8,11-", no_mmap, hck_delim);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -558,11 +837,11 @@ mod test {
     }
 
     #[rstest]
-    fn test_reorder1(#[values(true, false)] no_mmap: bool) {
+    fn test_reorder1(#[values(true, false)] no_mmap: bool, #[values(r"\s+")] hck_delim: &str) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "6,-4", no_mmap);
+        let opts = build_opts(&input_file, &output_file, "6,-4", no_mmap, hck_delim);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -579,12 +858,12 @@ mod test {
     }
 
     #[rstest]
-    fn test_reorder2(#[values(true, false)] no_mmap: bool) {
+    fn test_reorder2(#[values(true, false)] no_mmap: bool, #[values(r"\s+")] hck_delim: &str) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
         // 4-5 should not be repeated at the end and only written once.
-        let opts = build_opts(&input_file, &output_file, "3-,1,4-5", no_mmap);
+        let opts = build_opts(&input_file, &output_file, "3-,1,4-5", no_mmap, hck_delim);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -605,16 +884,19 @@ mod test {
 
     #[rstest]
     #[rustfmt::skip::macros(vec)]
-    fn test_read_single_values_not_regex(#[values(true, false)] no_mmap: bool) {
+    fn test_read_single_values_not_regex(
+        #[values(true, false)] no_mmap: bool,
+        #[values("    ", " ")] hck_delim: &str,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "1", no_mmap);
+        let opts = build_opts_not_regex(&input_file, &output_file, "1", no_mmap, hck_delim);
         let data = vec![
             vec!["a", "b", "c"],
             vec!["1", "2", "3"],
         ];
-        write_file(&input_file, data, FOURSPACE);
+        write_file(&input_file, data, hck_delim);
         run_wrapper(&input_file, &output_file, &opts);
         let filtered = read_tsv(output_file);
 
@@ -622,13 +904,16 @@ mod test {
     }
 
     #[rstest]
-    fn test_read_several_single_values_not_regex(#[values(true, false)] no_mmap: bool) {
+    fn test_read_several_single_values_not_regex(
+        #[values(true, false)] no_mmap: bool,
+        #[values("    ", " ")] hck_delim: &str,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "1,3", no_mmap);
+        let opts = build_opts_not_regex(&input_file, &output_file, "1,3", no_mmap, hck_delim);
         let data = vec![vec!["a", "b", "c"], vec!["1", "2", "3"]];
-        write_file(&input_file, data, FOURSPACE);
+        write_file(&input_file, data, hck_delim);
         run_wrapper(&input_file, &output_file, &opts);
         let filtered = read_tsv(output_file);
 
@@ -638,14 +923,15 @@ mod test {
     #[rstest]
     fn test_read_several_single_values_with_invalid_utf8_not_regex(
         #[values(true, false)] no_mmap: bool,
+        #[values("    ", " ")] hck_delim: &str,
     ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "1,3", no_mmap);
+        let opts = build_opts_not_regex(&input_file, &output_file, "1,3", no_mmap, hck_delim);
         let bad_str = unsafe { String::from_utf8_unchecked(b"a\xED\xA0\x80z".to_vec()) };
         let data = vec![vec![bad_str.as_str(), "b", "c"], vec!["1", "2", "3"]];
-        write_file(&input_file, data, FOURSPACE);
+        write_file(&input_file, data, hck_delim);
         run_wrapper(&input_file, &output_file, &opts);
         let filtered = read_tsv(output_file);
 
@@ -653,13 +939,16 @@ mod test {
     }
 
     #[rstest]
-    fn test_read_single_range_not_regex(#[values(true, false)] no_mmap: bool) {
+    fn test_read_single_range_not_regex(
+        #[values(true, false)] no_mmap: bool,
+        #[values("    ", " ")] hck_delim: &str,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "2-", no_mmap);
+        let opts = build_opts_not_regex(&input_file, &output_file, "2-", no_mmap, hck_delim);
         let data = vec![vec!["a", "b", "c", "d"], vec!["1", "2", "3", "4"]];
-        write_file(&input_file, data, FOURSPACE);
+        write_file(&input_file, data, hck_delim);
         run_wrapper(&input_file, &output_file, &opts);
         let filtered = read_tsv(output_file);
 
@@ -667,16 +956,19 @@ mod test {
     }
 
     #[rstest]
-    fn test_read_serveral_range_not_regex(#[values(true, false)] no_mmap: bool) {
+    fn test_read_serveral_range_not_regex(
+        #[values(true, false)] no_mmap: bool,
+        #[values("    ", " ")] hck_delim: &str,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "2-4,6-", no_mmap);
+        let opts = build_opts_not_regex(&input_file, &output_file, "2-4,6-", no_mmap, hck_delim);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
         ];
-        write_file(&input_file, data, FOURSPACE);
+        write_file(&input_file, data, hck_delim);
         run_wrapper(&input_file, &output_file, &opts);
         let filtered = read_tsv(output_file);
 
@@ -687,16 +979,19 @@ mod test {
     }
 
     #[rstest]
-    fn test_read_mixed_fields1_not_regex(#[values(true, false)] no_mmap: bool) {
+    fn test_read_mixed_fields1_not_regex(
+        #[values(true, false)] no_mmap: bool,
+        #[values("    ", " ")] hck_delim: &str,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "2,4-", no_mmap);
+        let opts = build_opts_not_regex(&input_file, &output_file, "2,4-", no_mmap, hck_delim);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
         ];
-        write_file(&input_file, data, FOURSPACE);
+        write_file(&input_file, data, hck_delim);
         run_wrapper(&input_file, &output_file, &opts);
         let filtered = read_tsv(output_file);
 
@@ -707,16 +1002,19 @@ mod test {
     }
 
     #[rstest]
-    fn test_read_mixed_fields2_not_regex(#[values(true, false)] no_mmap: bool) {
+    fn test_read_mixed_fields2_not_regex(
+        #[values(true, false)] no_mmap: bool,
+        #[values("    ", " ")] hck_delim: &str,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "-4,7", no_mmap);
+        let opts = build_opts_not_regex(&input_file, &output_file, "-4,7", no_mmap, hck_delim);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
         ];
-        write_file(&input_file, data, FOURSPACE);
+        write_file(&input_file, data, hck_delim);
         run_wrapper(&input_file, &output_file, &opts);
         let filtered = read_tsv(output_file);
 
@@ -727,11 +1025,14 @@ mod test {
     }
 
     #[rstest]
-    fn test_read_no_delimis_found_not_regex(#[values(true, false)] no_mmap: bool) {
+    fn test_read_no_delimis_found_not_regex(
+        #[values(true, false)] no_mmap: bool,
+        #[values("    ", " ")] hck_delim: &str,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "-4,7", no_mmap);
+        let opts = build_opts_not_regex(&input_file, &output_file, "-4,7", no_mmap, hck_delim);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -746,16 +1047,19 @@ mod test {
     }
 
     #[rstest]
-    fn test_read_over_end_not_regex(#[values(true, false)] no_mmap: bool) {
+    fn test_read_over_end_not_regex(
+        #[values(true, false)] no_mmap: bool,
+        #[values("    ", " ")] hck_delim: &str,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "-4,8,11-", no_mmap);
+        let opts = build_opts_not_regex(&input_file, &output_file, "-4,8,11-", no_mmap, hck_delim);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
         ];
-        write_file(&input_file, data, FOURSPACE);
+        write_file(&input_file, data, hck_delim);
         run_wrapper(&input_file, &output_file, &opts);
         let filtered = read_tsv(output_file);
 
@@ -767,16 +1071,19 @@ mod test {
     }
 
     #[rstest]
-    fn test_reorder1_not_regex(#[values(true, false)] no_mmap: bool) {
+    fn test_reorder1_not_regex(
+        #[values(true, false)] no_mmap: bool,
+        #[values("    ", " ")] hck_delim: &str,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "6,-4", no_mmap);
+        let opts = build_opts_not_regex(&input_file, &output_file, "6,-4", no_mmap, hck_delim);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
         ];
-        write_file(&input_file, data, FOURSPACE);
+        write_file(&input_file, data, hck_delim);
         run_wrapper(&input_file, &output_file, &opts);
         let filtered = read_tsv(output_file);
 
@@ -788,17 +1095,20 @@ mod test {
     }
 
     #[rstest]
-    fn test_reorder2_not_regex(#[values(true, false)] no_mmap: bool) {
+    fn test_reorder2_not_regex(
+        #[values(true, false)] no_mmap: bool,
+        #[values("    ", " ")] hck_delim: &str,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
         // 4-5 should not be repeated at the end and only written once.
-        let opts = build_opts_not_regex(&input_file, &output_file, "3-,1,4-5", no_mmap);
+        let opts = build_opts_not_regex(&input_file, &output_file, "3-,1,4-5", no_mmap, hck_delim);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
         ];
-        write_file(&input_file, data, FOURSPACE);
+        write_file(&input_file, data, hck_delim);
         run_wrapper(&input_file, &output_file, &opts);
         let filtered = read_tsv(output_file);
 
@@ -811,4 +1121,112 @@ mod test {
             ]
         );
     }
+
+    /// Tests from users
+    #[rstest]
+    fn test_reorder_no_split_found(
+        #[values(true, false)] no_mmap: bool,
+        #[values("    ", " ")] hck_delim: &str,
+    ) {
+        let tmp = TempDir::new().unwrap();
+        let input_file = tmp.path().join("input.txt");
+        let output_file = tmp.path().join("output.txt");
+        // 4-5 should not be repeated at the end and only written once.
+        let opts = build_opts_not_regex(&input_file, &output_file, "3-,1,4-5", no_mmap, hck_delim);
+        let data = vec![
+            vec!["a", "b", "c", "d", "e", "f", "g"],
+            vec!["1", "2", "3", "4", "5", "6", "7"],
+        ];
+        write_file(&input_file, data, "-");
+        run_wrapper(&input_file, &output_file, &opts);
+        let filtered = read_tsv(output_file);
+
+        // columns past end in fields are ignored
+        assert_eq!(filtered, vec![vec!["a-b-c-d-e-f-g"], vec!["1-2-3-4-5-6-7"]]);
+    }
+
+    /// Tests from users
+    #[rstest]
+    fn test_reorder_no_split_found_regex(
+        #[values(true, false)] no_mmap: bool,
+        #[values("    ", " ")] hck_delim: &str,
+    ) {
+        let tmp = TempDir::new().unwrap();
+        let input_file = tmp.path().join("input.txt");
+        let output_file = tmp.path().join("output.txt");
+        // 4-5 should not be repeated at the end and only written once.
+        let opts = build_opts(&input_file, &output_file, "3-,1,4-5", no_mmap, hck_delim);
+        let data = vec![
+            vec!["a", "b", "c", "d", "e", "f", "g"],
+            vec!["1", "2", "3", "4", "5", "6", "7"],
+        ];
+        write_file(&input_file, data, "---");
+        run_wrapper(&input_file, &output_file, &opts);
+        let filtered = read_tsv(output_file);
+
+        // columns past end in fields are ignored
+        assert_eq!(
+            filtered,
+            vec![
+                vec!["a---b---c---d---e---f---g"],
+                vec!["1---2---3---4---5---6---7"]
+            ]
+        );
+    }
+
+    #[rstest]
+    fn test_issue_12_with_regex(
+        #[values(true, false)] no_mmap: bool,
+        #[values(r"\s+")] hck_delim: &str,
+    ) {
+        let tmp = TempDir::new().unwrap();
+        let input_file = tmp.path().join("input.txt");
+        let output_file = tmp.path().join("output.txt");
+        // 4-5 should not be repeated at the end and only written once.
+        let opts = build_opts(&input_file, &output_file, "2,3,4-", no_mmap, hck_delim);
+        let data = vec![
+            vec!["a", "b", "c", "d", "e", "f", "g"],
+            vec!["1", "2", "3", "4", "5", "6", "7"],
+        ];
+        write_file(&input_file, data, "  ");
+        run_wrapper(&input_file, &output_file, &opts);
+        let filtered = read_tsv(output_file);
+
+        // columns past end in fields are ignored
+        assert_eq!(
+            filtered,
+            vec![
+                vec!["b", "c", "d", "e", "f", "g"],
+                vec!["2", "3", "4", "5", "6", "7"]
+            ]
+        );
+    }
+
+    #[rstest]
+    fn test_issue_12_no_regex(
+        #[values(true, false)] no_mmap: bool,
+        #[values("    ", " ")] hck_delim: &str,
+    ) {
+        let tmp = TempDir::new().unwrap();
+        let input_file = tmp.path().join("input.txt");
+        let output_file = tmp.path().join("output.txt");
+        // 4-5 should not be repeated at the end and only written once.
+        let opts = build_opts(&input_file, &output_file, "2,3,4-", no_mmap, hck_delim);
+        let data = vec![
+            vec!["a", "b", "c", "d", "e", "f", "g"],
+            vec!["1", "2", "3", "4", "5", "6", "7"],
+        ];
+        write_file(&input_file, data, hck_delim);
+        run_wrapper(&input_file, &output_file, &opts);
+        let filtered = read_tsv(output_file);
+
+        // columns past end in fields are ignored
+        assert_eq!(
+            filtered,
+            vec![
+                vec!["b", "c", "d", "e", "f", "g"],
+                vec!["2", "3", "4", "5", "6", "7"]
+            ]
+        );
+    }
 }
